VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CSettings"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
'Application settings class reads/write settings to Settings.ini in the application's path
'If settings.ini is not writable, the registry is used instead.

Private Const mcSection = "Settings"

Private Declare Function GetPrivateProfileString Lib "kernel32" Alias "GetPrivateProfileStringA" (ByVal sSection As String, ByVal sKeyName As String, ByVal sDefault As String, ByVal sReturned As String, ByVal nSize As Long, ByVal sFileName As String) As Long
Private Declare Function WritePrivateProfileString Lib "kernel32" Alias "WritePrivateProfileStringA" (ByVal sSection As String, ByVal sKeyName As String, ByVal sValue As String, ByVal sFileName As String) As Long
Private Declare Function WritePrivateProfileSection Lib "kernel32" Alias "WritePrivateProfileSectionA" (ByVal sSection As String, ByVal sData As String, ByVal sFileName As String) As Long

Private mbSaveToINI As Boolean
Private msAppName As String
Private msIniFile As String

'Returns True if settings are being read/written to the INI file
Public Property Get SaveToINI() As Boolean
    SaveToINI = mbSaveToINI
End Property

'If bValus is TRUE, saves settings to an INI file, if false, saves to registry
'If INI file is not writeable, saves to registry regardless of bValue
Public Property Let SaveToINI(bValue As Boolean)
    Dim bSuccess As Boolean
    
    On Error Resume Next   'DeleteSetting causes an error if the key doesn't exist
    
    If bValue = True Then
        bSuccess = INIWriteSetting("SaveToINI", CLng(True))
        If bSuccess = True Then
            VBA.DeleteSetting App.Title, mcSection, "SaveToINI"
            mbSaveToINI = True
            Exit Property
        End If
    End If
    
    'If bValue=False or write to INI failed, save to registry
    VBA.SaveSetting App.Title, mcSection, "SaveToINI", CLng(False)
    mbSaveToINI = False
    
    Exit Property

End Property


'Read a setting. If the setting is not saved, the default value is returned
Public Function GetSetting(sName As String, Optional vDefault As Variant = vbNullString) As Variant
    If SaveToINI = True Then
        GetSetting = INIGetSetting(sName, vDefault)
    Else
        GetSetting = VBA.GetSetting(msAppName, mcSection, sName, vDefault)
    End If
End Function

'
Public Function SaveSetting(sName As String, vValue As Variant)
    
    If TypeName(vValue) = "Boolean" Then vValue = CLng(vValue)
    
    If SaveToINI = True Then
        INIWriteSetting sName, vValue
    Else
        VBA.SaveSetting msAppName, mcSection, sName, vValue
    End If
End Function

'Delete the specified setting, or if no setting is specified, delete all settings
Public Function DeleteSetting(Optional sKey As String = vbNullString)

    If SaveToINI = True Then
        If Len(sKey) > 0 Then   'If no key specified, delete entire section
            INIWriteSetting sKey, vbNullString
        Else
            WritePrivateProfileSection mcSection, vbNullString, msIniFile
        End If
    Else
        If Len(sKey) > 0 Then   'If no key specified, delete entire section
            VBA.DeleteSetting msAppName, mcSection, sKey
        Else
            VBA.DeleteSetting msAppName, mcSection
        End If
    End If
End Function

' Returns a string value from the specified INI file
Private Function INIGetSetting(sKeyName As String, ByVal sDefault As String) As String
  Dim sBuffer As String * 256
  Dim nSize As Integer

  On Error GoTo ErrorHandler
  nSize = GetPrivateProfileString(mcSection, sKeyName, sDefault, sBuffer, 256, msIniFile)
  INIGetSetting = Left$(sBuffer, nSize)
  Exit Function
  
ErrorHandler:
  MsgBox "Error: " & Err.Number & ". " & Err.Description, , "INIGetSetting"
End Function


'Writes the specified value to the specified INI file
'Returns True if successful, False otherwise
Private Function INIWriteSetting(sKeyName As String, ByVal sValue As String) As Boolean
    Dim nStatus As Integer
    
    On Error GoTo ErrorHandler
    
    If Len(sValue) > 1 Then
        If Left(sValue, 1) = """" And Right(sValue, 1) = """" Then
            sValue = """" + sValue + """"
        End If
    End If
    
    nStatus = WritePrivateProfileString(mcSection, sKeyName, sValue, msIniFile)
    INIWriteSetting = (nStatus <> 0)
    Exit Function
    
ErrorHandler:
    MsgBox "Error: " & Err.Number & ". " & Err.Description, , "INIWriteSetting"
End Function


Private Sub Class_Initialize()
    msAppName = App.Title
    msIniFile = App.Path & "\Settings.ini"
    SaveToINI = VBA.GetSetting(msAppName, mcSection, "SaveToINI", True) ' FileExists(msIniFile)
End Sub

Private Sub Class_Terminate()
    SaveSetting "SaveToINI", SaveToINI  'Resave in case settings were deleted with DeleteSetting()
End Sub

